' Audio Trip Player: Based on ASM Wav Player
' Version 1.0
' Guillermo Pedraza
'
'  Notes
'     If software is to be placed on eeprom set "bootLoader = 1" so it will load any updates on SD card,  If file is going to be placed on
'     SD card then bootLoader MUST bet set to 0 or you will enter an infinite loop.
'  Changes to existing objects:
'       WaveEngine:
'               Added declick code to ramp up  volume to reduce click and pop when playing new audio. This was found in the parallax forums. Pop on power up still happens it happens when audio pins are set to output haven't found a way to elimated this.
'                
'       spdifOUt:
'               spdif.setPause   ' Makes spdifOut Engine wait.  Since it takes time to load wave files this stops the engine from sending invalid data, causing weird noises.
'               spdif.clearPause  ' Resumes spdif Engine
'       fsrwFemto,sdspiFemto:
'               sd.psave         ' Saves current file position to allow a quicker reload of wav file, used to play loop files
'               sd.preset        ' Loads previously saved file position 
'               
'
'  SOURCES
'  Original Wav Player code : 
'  ASM WAV Player Ver. 1b  (Plays only stereo, 16-bit PCM WAV files from SD card)
'  Copyright 2007 Raymond Allen
'  Settings for Demo Board Audio Output:  Right Pin# = 10, Left Pin# = 11   , VGA base=Pin16, TV base=Pin12
'  Rev.B:  21Dec07 Fixed pin assignment bug.  



CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000       '80 MHz

   buffSize = 256
   SPDIF_PIN     = 16

   spiDO     = 0
   spiClk    = 1
   spiDI     = 2
   spiCS     = 3

   bootLoader = 0
   
VAR
    
    long parameter1 'to pass @buff1 to ASM
    long parameter2  'to pass @buff2 to ASM
    long parameter3  'to pass sample rate to ASM
    long parameter4  'to pass #samples to ASM
    long doPlay
    long doQuit
    
    long buff1[buffSize]
    long buff2[buffSize]
    byte Header[44]
    byte tbuffer[50]
    long tlong
    
    long stack[100]
    byte playList[10]
    byte playPos
    byte actions[200]
    byte doLoop
    byte playLoop      
    byte wait
    byte doSPDIF
    byte sndExists[4]
    long ioControl[2]
     
OBJ
    SD  : "fsrwFemto"
    text :  "tv_text"
    'text : "audioTripGraphics"
    spdif : "spdifOut"
 
     
 
PUB Main|n,i,j, SampleRate,Samples,startTime,endTime,s,t,d,sw

  dira[18]:=0  
  dira[19]:=0
  dira[20]:=0
  dira[21]:=0
  dira[15]:=1
  dira[17]:=1

  '#if useGraphics
  '  text.setcolors(@mypalette)   
  '  setColor(0)
  '  text.init  
  '#if Regular text
  text.start(12)    'Start the VGA/TV text driver (uses another cog)

  ' Initialize Variables
  isPlaying:=0
  playLoop:=0            
  doSPDIF:=0

  repeat i from 0 to 3
    sndExists[i]:=0    

  'Initialize SD Card, If not found wait for it  (NOTE:  Only plays stereo, 16-bit PCM WAV Files !!!!!!!!!!!!!)
  text.str(String("Audio Trip v1.0",13))
  sd.start(@ioControl)
  i:=0
  dira[SPDIF_PIN]~~
  repeat
    if \sd.mount(spiDO,spiClk,spiDI,spiCS) < 0            ' Attempt to mount SD card
      text.str(String("Insert SD Card",13))
      waitcnt(cnt+10_000_000)          ' Blink LED in TOS Link
      !outa[SPDIF_PIN]
    else
      i:=1
  until i>0
                                
   
  ' Search for files on SD Card needed   
  text.str(STRING("SD Card Mounted",13))
  text.str(STRING("Files:",13))
  s:=0
  sw:=0
  
  sd.opendir
  repeat 
    s:=sd.nextFile(@tbuffer)
    if s <> 0 
        ' do nothing...last file
    elseif strcomp(@boottxt,@tbuffer)
       if bootLoader==1
        text.str(String("Boot From SD",13))
        waitcnt(cnt+90_000_000) 
        if \sd.popen(@tbuffer,"r")                           
             text.str(String("Boot Failed",13))
             abort                             
        sd.bootSDCard
    elseif strcomp(@looptxt,@tbuffer)
       text.str(STRING("Found LOOP",13))
       playLoop:=1
    elseif strcomp(@snd1txt,@tbuffer)
         text.str(STRING("Found SND1",13))
         sndExists[0]:=1  
    elseif strcomp(@snd2txt,@tbuffer)
         text.str(STRING("Found SND2",13))
         sndExists[1]:=1 
    elseif strcomp(@snd3txt,@tbuffer)
         text.str(STRING("Found SND3",13))
         sndExists[2]:=1 
    elseif strcomp(@snd4txt,@tbuffer)
         text.str(STRING("Found SND4",13))
         sndExists[3]:=1 
    elseif strcomp(@spdiftxt,@tbuffer)
         text.str(STRING("Found SPDIF"))
         doSPDIF:=1              
    sw++
  until s <> 0
       
  
  text.str(String("System Ready",13)) 
  SampleRate:=44100                  ' Fill with dummy values until actual audio file playes
  Samples:=14174208    
  parameter1:=@buff1[0]
  parameter2:=@buff2[0]
  parameter3:=CLKFREQ/SampleRate  '#clocks between samples'1814'for 44100ksps,  5000 'for 16ksps
  parameter4:=0'Samples
  doPlay:=0
  doQuit:=0

  ' Start appropriate sound Engine.
  ' If spdif.txt is found on SD Card enable digital output via SPDIF and coaxial
  if doSPDIF == 0
   COGNEW(@ASMWAV,@parameter1)
   COGNEW(WavEngine,@stack)
   dira[SPDIF_PIN]~~
   !outa[SPDIF_PIN]
  else
    spdif.setBuffer(@buff1, constant(buffSize * 2))
    spdif.start(SPDIF_PIN) 
    COGNEW(WavSPDIFEngine,@stack) 

  
  sw:=0
  wait:=0

  ' Start permanent loop. Wait for trigger to be activated to start sound        
  repeat
     if isPlaying==0 and DoPlay==0    
        if INA[18]==0 and sndExists[0]==1       
             isPlaying:=1
             Play(2)
             'DEMO1
        elseif INA[19]==0 and sndExists[1]==1
             isPlaying:=1
             Play(3)
        elseif INA[20]==0 and sndExists[2]==1
             isPlaying:=1
             Play(4)
        elseif INA[21]==0  and sndExists[3]==1    
             isPlaying:=1
             Play(5)
               
PUB Play(song) | i 
   playList[playPos]:=song
   playPos++
   if playPos>9
    playPos:=0
   
PUB nl
  text.out(13)
  text.out(10)


' Wav SPDIF Engine...based on Micah Dowty's spdifOut object.
' Wait for audio data, if audio loop is activated play until interrupted by trigger 
       
PRI WavSPDIFEngine | n,i,j, SampleRate,Samples,s 
   s:=0         
repeat
  if isPlaying==1
    text.str(String("Audio Complete",13))          
    isPlaying:=0
    
  s:=0
  repeat while s==0
     repeat i from 0 to 9   
      if playList[i] > 0
       s := playList[i]~
       wait:=0
       quit
     if playLoop==1 and s==0 and wait==0 
        s:=1
        isPlaying:=1
        doLoop:=1
 '            
  if s==1 
    i:=sd.popen(string("loop.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!         
  elseif s==2
    i:=sd.popen(string("SND1.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  elseif s==3
    i:=sd.popen(string("SND2.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  elseif s==4
    i:=sd.popen(string("SND3.wav"), "r")                  '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  else
    i:=sd.popen(string("SND4.wav"),"r")
  if (i<>0)
    text.str(string("File not found!"))
    repeat
    
  doQuit:=0
  doPlay:=1

  sd.psave   ' Save current wave file position. For faster lookup when playing loop.  
 repeat
  
  j:=buffsize*4
  spdif.clearPause    
  repeat
    ' Wait until the driver is using bufB, then read bufA
    if j==constant(buffsize*4)
      repeat until spdif.getCount & BUFFSIZE
      j:=sd.pread(@buff1, constant(BUFFSIZE * 4))
    elseif doLoop==1
       sd.preset
       j:=sd.pread(@buff1, constant(BUFFSIZE * 4))
                                
    ' Now the opposite...
    if j==constant(buffsize*4)
      repeat while spdif.getCount & BUFFSIZE
      j:=sd.pread(@buff2, constant(BUFFSIZE * 4))
    elseif doLoop==1
      sd.preset
      j:=sd.pread(@buff2, constant(BUFFSIZE * 4))
                    
    if doQuit<>0
      j:=0

    ' If playing loop check for any triggers.
    if doLoop ==1
      if (INA[18]==0 and sndExists[0]==1) OR (INA[19]==0 and sndExists[1]==1) or (INA[20]==0 and sndExists[2]==1) or (INA[21]==0 and sndExists[3]==1)
       doLoop:=0
       doQuit:=1
       doPlay:=0
       wait:=1
       j:=0

       
  until (j<>constant(buffsize*4))
  spdif.setPause                      
 until doLoop ==0
   doPlay:=0
  'must have reached the end of the file, so close it
   sd.pclose

PUB displayString(row, col, s)
  setPosition(row, col)
  text.str(s)

PUB setPosition(row, col)
  if row => 0
    text.out($B)
    text.out(row)
  if col => 0
    text.out($A)
    text.out(col)

PUB setColor(c)
  text.out($C)                                  
  text.out(c)

' Wav Engine...based on Raymond Allen's  ASM Wav Player
' Wait for audio data, if audio loop is activated play until interrupted by trigger  
PRI WavEngine | n,i,j, SampleRate,Samples,s 
  'Keep filling buffers until end of file
  ' note:  using alternating buffers to keep data always at the ready...
   s:=0

repeat
  if isPlaying==1
    text.str(String("Audio Complete",13))          
    isPlaying:=0

    
  s:=0
  repeat while s==0
     repeat i from 0 to 9   
      if playList[i] > 0
       s := playList[i]~
       wait:=0
       quit
     if playLoop==1 and s==0 and wait==0 
        s:=1
        isPlaying:=1
        doLoop:=1
 repeat      
  if s==1 
    i:=sd.popen(string("loop.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!         
  elseif s==2
    i:=sd.popen(string("carrys.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  elseif s==3
    i:=sd.popen(string("SND3.wav"), "r")                '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  elseif s==4
    i:=sd.popen(string("SND4.wav"), "r")                  '   <---------  Change .wav filename here !!!!!!!!!!!!!!
  else
    i:=sd.popen(string("hell.wav"),"r")
  if (i<>0)
    text.str(string("File not found!"))
    repeat
  if s > 1 
    text.str(STRING("Play SND ",13))
  
  'ignoring much file header (so you better have the format right!)    stereo, 16-bit PCM WAV Files !!!!!!!!!!!!!!!!
  'See here for header format:  http://ccrma.stanford.edu/CCRMA/Courses/422/projects/WaveFormat/
  i:=sd.pread(@Header, 44) 'read data words to input stereo buffer
  'Get sample rate from header
  SampleRate:=Header[27]<<24+Header[26]<<16+Header[25]<<8+Header[24]
  'get # samples from header
  Samples:=Header[43]<<24+Header[42]<<16+Header[41]<<8+Header[40]
  Samples:=Samples>>2
  if doLoop==1
    Samples:=-1                 

  parameter1:=@buff1[0]
  parameter2:=@buff2[0]
  parameter3:=CLKFREQ/SampleRate  '#clocks between samples'1814'for 44100ksps,  5000 'for 16ksps
  parameter4:=Samples
  doQuit:=0
  doPlay:=1

  longfill(@buff1, 0, buffSize)
  longfill(@buff2, 0, buffSize)

  n:=buffSize-1
  j:=buffsize*4   'number of bytes to read
  
  repeat while (j==buffsize*4)  'repeat until end of file
    if (buff1[n]==0)
      j:=sd.pread(@buff1, buffSize*4) 'read data words to input stereo buffer   

    if (buff2[n]==0)
      j:=sd.pread(@buff2, buffSize*4) 'read data words to input stereo buffer

    if doQuit<>0
      j:=0

   ' Check for Triggers during loop audio, if found stop sound and play requested sound 
    if doLoop ==1
      if (INA[18]==0 and sndExists[0]==1) OR (INA[19]==0 and sndExists[1]==1) or (INA[20]==0 and sndExists[2]==1) or (INA[21]==0 and sndExists[3]==1)
       doLoop:=0
       doQuit:=1
       doPlay:=0
       wait:=1
       j:=0  
                        
 until doLoop ==0
  'must have reached the end of the file, so close it
  sd.pclose
 
 
DAT
  ORG 0
ASMWAV

setup                                                     
        mov     dira,Port_Pins          'Setup output pins
        mov     ctra,Right_ctra         'Setup Right Audio Channel
        mov     ctrb,Left_ctra          'Setup Left Audio Channel 
        
        ' Fast lane....
        mov val, #0
:declick                            ' ramp up the audio DC output voltage from 0 to midway (0.5 VDD)
        'waitcnt nexttime, delta
        mov nexttime, cnt
        add nexttime, delay

        waitcnt nexttime, delta
       
        mov frqa, val
        mov frqb, val
        add val,fadeval
        cmp val,midval wz          ' midway yet?
        if_ne jmp #:declick

readhub 
                'load input parameters from hub to cog given address in par
        movd    :par,#pData1             
        mov     x,par
        mov     y,#6  'input 4 parameters
:par    rdlong  0,x
        add     :par,dlsb
        add     x,#4
        djnz    y,#:par
        ' End of Read Parameters

        tjz nDoPlay, #readhub
reloop

        'Wait for SPIN to fill table
        MOV WaitCount, CNT
        ADD WaitCount,BigWait
        WAITCNT WaitCount,#0
        'setup loop table
        MOV LoopCount,SizeBuff  
        'ROR LoopCount,#1    'for stereo
        MOV pData,pData1
        MOV nTable,#1
        'setup loop counter
        MOV WaitCount, CNT
        ADD WaitCount,dRate

      '  mov x,par             ' Write to doplay
      '  add x,#16
      '  wrlong ctr,x

        SUBS nSamples,#500'sampleDec
        SUBS nSamples,#400'sampleDec
        mov x,par
        add x,#20
MainLoop
        RDlong _doQuit,x
        tjnz _doQuit, #Done

       '; wrlong nSamples,x

        SUB nSamples,#1
        CMP nSamples,#0 wz
        IF_Z JMP #Done
        waitcnt WaitCount,dRate

        RDLONG Right,pData
        ADD Right,twos      'Going to cheat a bit with the LSBs here...  Probably shoud fix this!    
        MOV FRQA,Right
        ROL Right,#16       '16 LSBs are left channel...
        MOV FRQB,Right
        WRLONG Zero,pData
        ADD pData,#4
        
        'loop
        DJNZ LoopCount,#MainLoop
        
        MOV LoopCount,SizeBuff        
        'switch table       ?
        CMP nTable,#1 wz
        IF_Z JMP #SwitchToTable2
SwitchToTable1
        MOV nTable,#1
        MOV pData,pData1
        JMP #MainLoop
SwitchToTable2
        MOV nTable,#2
        MOV pData,pData2
        JMP #MainLoop
Done   
       'now stop
        mov x,par
        add x,#20
        wrlong ONE,x
        
        mov frqa, midval
        mov frqb, midval

        ' Clear do loop
        mov x,par
        add x,#16
        wrlong Zero,x
        
        ' Wait for doPlay = 1
        jmp #readhub
        
        COGID thisCog
        COGSTOP thisCog          

' Setup Vars
Port_Pins     long      %00000000_00000000_00001100_00000000

                        '- CTR   PLL -------- BPIN   --- APIN
Right_ctra    long      %0_00110_000_00000000_000000_000_001010
Left_ctra     long      %0_00110_000_00000000_000000_000_001011


'Working variables
thisCog long 0
x       long 0
y       long 0
dlsb    long    1 << 9
BigWait long 100000
twos    long $8000_8000
        
'Loop parameters
nTable  long 0
WaitCount long 0
pData   long 0
LoopCount long 0
SizeBuff long buffsize
'Left    long 0
Right   long 0
Zero    long 0          

'setup parameters
DMaskR  long 0 'right output mask
OPinR   long 10 'right channel output pin #                        '   <---------  Change Right pin# here !!!!!!!!!!!!!!    
DMaskL  long 0 'left output mask 
OPinL   long 11 'left channel output pin #                         '   <---------  Change Left pin# here !!!!!!!!!!!!!!    
CountModeR long %00011000_00000000_00000000_00000000
CountModeL long %00011000_00000000_00000000_00000000


'input parameters

pData1   long 0 'Address of first data table        
pData2   long 0 'Address of second data table
dRate    long 5000  'clocks between samples
nSamples long 2000
nDoPlay  long 0
_doQuit  long 0

' Ramp vars
fadeval  long $8_0000
midval   long $8000_0000
delay    long 1_0000
nexttime res 1   
delta    long 0
val      long 0

ctr      long 2
Index    long 1000
Count    long 0
isPlaying     byte 0
audioStatus   long 0
aSamples      long 0
p             res 1
addDoPlay     long 0
sampleDec     long 1
ONE           res 1

mypalette               byte    $07,   $0A    '0    white / dark blue
                        byte    $07,   $BB    '1    white / red
                        byte    $9E,   $9B    '2   yellow / brown
                        byte    $04,   $07    '3     grey / white
                        byte    $3D,   $3B    '4     cyan / dark cyan
                        byte    $6B,   $6E    '5    green / gray-green
                        byte    $BB,   $CE    '6      red / pink
                        byte    $3C,   $0A    '7     cyan / blue
                        
looptxt       byte   "LOOP.WAV",0
snd1txt       byte   "SND1.WAV",0     
snd2txt       byte   "SND2.WAV",0    
snd3txt       byte   "SND3.WAV",0     
snd4txt       byte   "SND4.WAV",0     
snd5txt       byte   "SND5.WAV",0
boottxt       byte   "AUDIO.BIN",0
spdiftxt      byte   "SPDIF.TXT",0